//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "VVetoPerf.h"

ClassImp(VetoPerf)

////////////////////////////////////////////////////////////////////////////////////
VP::VP(const string vdef, const bool vchannel, const int ncat){ 
////////////////////////////////////////////////////////////////////////////////////
  vetochannel=vchannel;
  if(vetochannel){// veto channels
    vetoname=vdef;
    vetofile="";
  }
  else{// veto file
    vetoname=GetFileName(vdef);
    vetofile=vdef;
  }
  n_cat=ncat;
  n_seguse = new int [n_cat];
  n_cveto = new int [n_cat];
  Reset();
}

////////////////////////////////////////////////////////////////////////////////////
void VP::Reset(void){ 
////////////////////////////////////////////////////////////////////////////////////
  dt=0.0;
  n_seg=0;
  for(int cat=0; cat<n_cat; cat++){
    n_seguse[cat]=0;
    n_cveto[cat]=0;
  }
}

////////////////////////////////////////////////////////////////////////////////////
VP::~VP(void){ 
////////////////////////////////////////////////////////////////////////////////////
  delete n_seguse;
  delete n_cveto;
}


////////////////////////////////////////////////////////////////////////////////////
VetoPerf::VetoPerf(const vector <double> aSnrThr){ 
////////////////////////////////////////////////////////////////////////////////////

  // timer starts
  time ( &timer_start );

  // output directory
  outdir=".";
  subtitle="";

  // SNR threshold
  if(aSnrThr.size()) snrthr=aSnrThr;
  else{
    snrthr.push_back(5.0);
    snrthr.push_back(8.0);
    snrthr.push_back(10.0);
    snrthr.push_back(20.0);
  }
  
  // veto numbers
  nveto=0;
  vp = new VP * [nveto];

  // ffl (none)
  SetFfl();

  // triggers
  TP = NULL;
  h1_snr = new TH1D();
  n_c=0;
  n_c_snr = new int [(int)snrthr.size()];
  for(int s=0; s<(int)snrthr.size(); s++) n_c_snr[s]=0;
  freqmin=-1.0;
  freqmax=1e20;
  
  // input segments
  InSeg= new Segments();
  UseSecondSegments();
}

////////////////////////////////////////////////////////////////////////////////////
VetoPerf::~VetoPerf(void){
////////////////////////////////////////////////////////////////////////////////////
  delete h1_snr;
  delete InSeg;
  delete n_c_snr;
  for(int v=0; v<nveto; v++) delete vp[v];
  delete vp;
}

////////////////////////////////////////////////////////////////////////////////////
bool VetoPerf::MakeVetoPerformance(const int aVetoIndex){
////////////////////////////////////////////////////////////////////////////////////
  if(TP==NULL){
    cerr<<"VetoPerf::MakeVetoPerformance: The trigger set must be defined first."<<endl;
    return false;
  }
  if(aVetoIndex<0||aVetoIndex>=nveto){
    cerr<<"VetoPerf::MakeVetoPerformance: veto index does not exist"<<endl;
    return false;
  }
  
  // Load veto segments
  Segments *vetoseg;
  if(vp[aVetoIndex]->vetochannel) vetoseg=GetSegmentsFromChannel(vp[aVetoIndex]->vetoname);
  else vetoseg = new Segments(vp[aVetoIndex]->vetofile);// segments are tagged to -1
  if(!vetoseg->GetStatus()){
    cerr<<"VetoPerf::MakeVetoPerformance: veto "<<vp[aVetoIndex]->vetoname<<" is corrupted"<<endl;
    return false;
  }

  // option to round segments
  if(usesec) vetoseg->MakeSecond();
  
  // print
  cout<<"VetoPerf::MakeVetoPerformance: Analysis of "<<vp[aVetoIndex]->vetoname<<"..."<<endl;
  
  // reset veto structure numbers
  vp[aVetoIndex]->Reset();
  
  // apply segment selection
  vetoseg->Intersect(InSeg);
  vp[aVetoIndex]->dt=vetoseg->GetLiveTime();
  vp[aVetoIndex]->n_seg=vetoseg->GetNsegments();

  // apply veto to trigger set
  double ct, csnr; int segn;
  for(int c=0; c<TP->GetNclusters(); c++){
    if(TP->GetClusterTag(c)<0) continue;// cluster is excluded
    
    // cluster peak time and SNR
    ct=TP->GetClusterTime(c);
    csnr=TP->GetClusterSNR(c);

    // activate cluster
    TP->SetClusterTag(c,1);
    
    // loop over SNR categories
    for(int s=0; s<(int)snrthr.size(); s++){
	
      if(csnr>=snrthr[s]){
	  
	// veto cluster?
	if(vetoseg->IsInsideSegment(ct,segn)){// cluster is vetoed
	  TP->SetClusterTag(c,0);// cluster is vetoed
	  vp[aVetoIndex]->n_cveto[s]+=1;
	  if(vetoseg->GetTag(segn)<s+1){ vp[aVetoIndex]->n_seguse[s]++; vetoseg->SetTag(segn,s+1); } // tag segment as used
	}	
      }
    }
  }
  
  // make collections
  for(int s=0; s<(int)snrthr.size(); s++) TP->SetCollectionUseClusters(s,1);// remove vetoed clusters
  TP->MakeCollections(-1,InSeg);
  
  // print
  cout<<"veto file: "<<vp[aVetoIndex]->vetofile<<endl;
  cout<<"veto dead time: "<<vp[aVetoIndex]->dt<<endl;
  cout<<"number of veto active segments: "<<vp[aVetoIndex]->n_seg<<endl;
  cout<<"number of vetoed clusters: ";
  for(int s=0; s<(int)snrthr.size(); s++) cout<<vp[aVetoIndex]->n_cveto[s]<<"/"<<n_c_snr[s]<<" ";
  cout<<endl;
 
  // print 2D plots
  stringstream tmpstream;
  tmpstream<<TP->GetNameConv()<<"-"+vp[aVetoIndex]->vetoname;
  TP->PrintCollectionPlot("freqtime");
  TP->DrawLegend();
  TP->Print(outdir+"/"+tmpstream.str()+"_freqtime.png");
  TP->PrintCollectionPlot("snrtime");
  TP->DrawLegend();
  TP->Print(outdir+"/"+tmpstream.str()+"_snrtime.png");
  TP->PrintCollectionPlot("snrfreq");
  TP->DrawLegend();
  TP->Print(outdir+"/"+tmpstream.str()+"_snrfreq.png");

  // print 1D plots
  TP->PrintPlot(h1_snr);
  TP->PrintPlot("snr",0,false);
  TP->DrawLegend();
  TP->Print(outdir+"/"+tmpstream.str()+"_snr.png");

  delete vetoseg;
  return true;
}

////////////////////////////////////////////////////////////////////////////////////
int VetoPerf::SetVetoes(const string aVetoFilePattern, const string aVetoChannelFile){
////////////////////////////////////////////////////////////////////////////////////
  if(aVetoChannelFile.compare("")&&ffldata==NULL){
    cerr<<"VetoPerf::SetVetoes: a FFL file must be provided first"<<endl;
    return 0;
  }
  
  // veto inputs: file or channel
  vector <string> vetodef;
  vector <bool> vetotype;

  // read veto files input
  if(aVetoFilePattern.compare("")){
    string ls;
    ls = "ls -U "+aVetoFilePattern;
    
    FILE *in;
    in = popen(ls.c_str(), "r");

    char filename[100000];
    while(fscanf(in,"%s",filename) != EOF){
      vetodef.push_back(filename);
      vetotype.push_back(false);
    }
    pclose(in);
  }

  // veto channels input
  string vchannel;
  if(aVetoChannelFile.compare("")){
    string chfilter="XXXXX";      
    ReadAscii *R = new ReadAscii(aVetoChannelFile,"s");
    for(int r=0; r<R->GetNRow(); r++){
      R->GetElement(vchannel, r, 0);
      chfilter+=" "+vchannel;
    }
    delete R;
    vector <string> chtmp;
    chtmp = ffldata->GetChannelList(chfilter);
    for(int c=0; c<(int)chtmp.size(); c++){
      vetodef.push_back(chtmp[c]);
      vetotype.push_back(true);
    }
    chtmp.clear();
  }
  
  // reset vetoes
  for(int v=0; v<nveto; v++) delete vp[v];
  delete vp;
  nveto=0;

  // init new veto structure
  nveto=(int)vetodef.size();
  vp= new VP * [nveto];
  for(int v=0; v<nveto; v++)
    vp[v] = new VP(vetodef[v],vetotype[v],(int)snrthr.size());

  // clear
  vetodef.clear();
  vetotype.clear();

  // check name unicity
  for(int v=0; v<nveto; v++){
    for(int vv=v+1; vv<nveto; vv++){
      if(!(vp[v]->vetoname).compare(vp[vv]->vetoname)){
	cerr<<"VetoPerf::SetVetoes: veto names must be unique: "<<vp[v]->vetoname<<endl;
	for(int v=0; v<nveto; v++) delete vp[v];
	nveto=0;
      }
    }
  }

  return nveto;
}


////////////////////////////////////////////////////////////////////////////////////
bool VetoPerf::SetTriggers(TriggerPlot* aTriggers, Segments *aInSeg){
////////////////////////////////////////////////////////////////////////////////////

  // trigger plot structure
  if(!aTriggers->GetStatus()){
    cerr<<"VetoPerf::SetTriggers: the input trigger structure is corrupted"<<endl;
    return false;
  }
  if(aTriggers->GetNCollections()!=(int)snrthr.size()){
    cerr<<"VetoPerf::SetTriggers: the input trigger structure must match the number of SNR thresholds ("<<snrthr.size()<<")"<<endl;
    return false;
  }

  // save pointer to triggers
  TP = aTriggers;
  
  // copy input segments
  delete InSeg;
  InSeg= new Segments(TP->GetSegments()->GetStarts(),TP->GetSegments()->GetEnds());
  if(aInSeg!=NULL&&aInSeg->GetStatus()) InSeg->Intersect(aInSeg);

  // clusters
  n_c = 0;
  for(int s=0; s<(int)snrthr.size(); s++) n_c_snr[s]=0;

  // use input segment and frequency range to select clusters
  for(int c=0; c<TP->GetNclusters(); c++){
    if(!InSeg->IsInsideSegment(TP->GetClusterTime(c))) TP->SetClusterTag(c,-1);
    if(TP->GetClusterFrequency(c)<freqmin) TP->SetClusterTag(c,-1);
    if(TP->GetClusterFrequency(c)>freqmax) TP->SetClusterTag(c,-1);
    if(TP->GetClusterTag(c)>=0) n_c++;
  }

  // frequency range for trigger plots
  for(int s=0; s<(int)snrthr.size(); s++){
    if(TP->GetFrequencyMax()>freqmax) TP->GetCollectionSelection(s)->SetFrequencyMax(freqmax);
    if(TP->GetFrequencyMin()<freqmin) TP->GetCollectionSelection(s)->SetFrequencyMin(freqmin);
  }
  
  // make collections
  MakeCollections(TP);
  for(int s=0; s<(int)snrthr.size(); s++) n_c_snr[s]=TP->GetCollectionNClusters(s);
      

  // print trigger plots
  stringstream tmpstream;
  tmpstream<<TP->GetNameConv()<<"-all";
  TP->PrintCollectionPlot("freqtime");
  TP->DrawLegend();
  TP->Print(outdir+"/"+tmpstream.str()+"_freqtime.png");
  TP->PrintCollectionPlot("snrtime");
  TP->DrawLegend();
  TP->Print(outdir+"/"+tmpstream.str()+"_snrtime.png");
  TP->PrintCollectionPlot("snrfreq");
  TP->DrawLegend();
  TP->Print(outdir+"/"+tmpstream.str()+"_snrfreq.png");

  // get histos
  delete h1_snr; h1_snr = TP->GetTH1D("snr",0);
  h1_snr->SetFillColor(15);
  h1_snr->SetLineColor(1);

  return true;
}

////////////////////////////////////////////////////////////////////////////////////
bool VetoPerf::SetOutputDirectory(const string aOutdir){
////////////////////////////////////////////////////////////////////////////////////
  if(!IsDirectory(aOutdir)){
    cerr<<aOutdir<<" is not a directory"<<endl;
    return false;
  }
  outdir=aOutdir;
  return true;
}
  
////////////////////////////////////////////////////////////////////////////////////
void VetoPerf::MakeCollections(TriggerPlot *aTP){
////////////////////////////////////////////////////////////////////////////////////

  // SNR max
  double snrmax;
  if(aTP->GetCollectionSelection(0)->GetSNRMax()<50.0)         snrmax=50.0;
  else if(aTP->GetCollectionSelection(0)->GetSNRMax()<100.0)   snrmax=100.0;
  else if(aTP->GetCollectionSelection(0)->GetSNRMax()<1000.0)  snrmax=1000.0;
  else if(aTP->GetCollectionSelection(0)->GetSNRMax()<10000.0) snrmax=10000.0;
  else                                                         snrmax=100000.0;

  // time binning
  int gps_start=(int)(InSeg->GetStart(0));
  int gps_end=(int)(InSeg->GetEnd(InSeg->GetNsegments()-1));

  int ntbins=100;

  stringstream tmpstream;

  // remove time box
  aTP->SetTimeBox(false);
  
  // Apply plot selections
  for(int s=0; s<(int)snrthr.size(); s++){

    // snr thresholds
    aTP->GetCollectionSelection(s)->SetSNRRange(snrthr[s],snrmax);
    tmpstream<<"SNR \\ge "<<fixed<<setprecision(1)<<snrthr[s];
    aTP->SetCollectionLegend(s,tmpstream.str());
    tmpstream.str(""); tmpstream.clear();

    // time range
    aTP->GetCollectionSelection(s)->SetTimeRange(gps_start, gps_end);

    // time resolution (FIXME)
    aTP->GetCollectionSelection(s)->SetTimeResolution(ntbins);

    // use clusters
    aTP->SetCollectionUseClusters(s,0);

    // time format
    aTP->SetDateFormat(true);

    // set collection marker
    if(s) aTP->SetCollectionMarker(s,20, TMath::Max(0.3,(double)(s+1)/(double)(snrthr.size())));
    else aTP->SetCollectionMarker(s,1,1);
		       
    // set collection color
    aTP->SetCollectionColor(s,TP->GetColorPalette((int)((double)(s+1)/(double)(snrthr.size())*(double)(TP->GetNumberOfColors()))-2));
      
  }

  // make collections
  aTP->MakeCollections(-1,InSeg);

  return;
}
 
////////////////////////////////////////////////////////////////////////////////////
Segments* VetoPerf::GetSegmentsFromChannel(const string aChannelName){
////////////////////////////////////////////////////////////////////////////////////

  // start segment structure
  Segments* seg = new Segments();

  // no ffl
  if(ffldata==NULL){
    cerr<<"VetoPerf::GetSegmentsFromChannel: an ffl must be provided"<<endl;
    return seg;
  }

  // no livetime
  if(!InSeg->GetLiveTime()) return seg;


  // load frame file
  if(!ffldata->LoadFrameFile(InSeg->GetStart(0))) return seg;

  // channel sampling
  int chindex;
  int samplingrate = ffldata->GetChannelSampling(aChannelName,chindex);
  if(samplingrate<=0){
    cerr<<"VetoPerf::GetSegmentsFromChannel: cannot extract channel "<<aChannelName<<endl;
    return seg;
  }

  // data chunk duration
  int chunkduration=1;
  if(samplingrate<10) chunkduration=100000;
  else if(samplingrate<1000) chunkduration=1000;
  else if(samplingrate<10000) chunkduration=100;
  else chunkduration=10;

  
  // loop over channel data
  double gstart, gstop;
  int vsize;
  double *vdata;
  for(int s=0; s<InSeg->GetNsegments(); s++){

    // first data segment
    gstart=InSeg->GetStart(s);
    
    // get data
    while(gstart<InSeg->GetEnd(s)){
      
      gstop=TMath::Min(gstart+chunkduration,InSeg->GetEnd(s));
      vdata = ffldata->GetData(vsize,aChannelName,gstart,gstop,-1.0);

      // test data
      for(int d=0; d<vsize; d++)
	if(vdata[d]>0) seg->AddSegment(gstart+(double)d/(double)samplingrate, gstart+(double)(d+1)/(double)samplingrate);

      // next round
      gstart+=chunkduration;
    }
  }
  
  return seg;
}
